library ac_dart.server;

import 'addon_route_builder.dart';
export 'addon_route_builder.dart';
import 'package:atlassian_connect_host/atlassian_connect_host.dart';
export 'package:atlassian_connect_host/atlassian_connect_host.dart';
export 'package:atlassian_connect_host/host_registration_persistence.dart';

import 'dart:io';
import 'package:logging/logging.dart';
export 'package:atlassian_connect_jwt/atlassian_connect_jwt.dart';

import 'package:atlassian_connect_config/atlassian_connect_config.dart';
export 'package:atlassian_connect_config/atlassian_connect_config.dart';

import 'package:shelf/shelf_io.dart' as io;

typedef RouteInitialiser(AcDartRouteBuilder routeBuilder, String addonKey,
    Uri addonBaseUri);

// TODO: move into config so can auto register for non prod herokus (assuming that works)
final List<Uri> _autoRegisterHostUrls = [
        Uri.parse("http://localhost:1990/confluence"),
        Uri.parse("http://localhost:2990/jira")
      ];

final Logger _log = new Logger('AcDartServer');


/// starts the shelf io server calling the provided [initRoutes] function to
/// initialise the routes and using provided [configFactory] for creating the
/// configuration for this environment. It also initialises logging which can
/// be overriding by supplying your own [initLogging] function
void startServer(RouteInitialiser initRoutes, AcDartConfigFactory configFactory,
                 { void initLogging() : _initLogging }) {

  initLogging();
  
  final config = configFactory.config;

  final addonBaseUrl = config.addonBaseUrl;
  
  final hostService = new AcHostService(config.acHostRepo, config.addonHost);
  
  void _doAutoInstall() {
    _autoRegisterHostUrls.forEach((baseUrl) =>
        hostService.installAddonToHost(baseUrl));
  }
  final AcDartMainRouteBuilder routeBuilder = 
      new AcDartMainRouteBuilder(hostService, config.addonHost);

  initRoutes(routeBuilder, config.addonKey, addonBaseUrl);
  
  io.serve(routeBuilder.handler, InternetAddress.ANY_IP_V6, addonBaseUrl.port)
    .then((server) {
      print('Serving at http://${server.address.host}:${server.port}');
  
      if (config.installAddonOnStartup) {
        _log.info('installing addon...');
        _doAutoInstall();
      }
  
    });
}

void _initLogging() {
  Logger.root.level = Level.FINEST; // TODO: read from env
  Logger.root.onRecord.listen((LogRecord rec) {
    print('${rec.level.name}: ${rec.time}: ${rec.message}');
  });  
}
